#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# For those saying "BSD R.I.P": kill yourself! ;)

__version__ = 'finished'
__description__     = 'Package for easy notes management'
__author__    = "GFORGX (gforgx@gmail.com)"
__copyright__ = "Copyright 2008, GFORGX"
__license__ = "BSD (revised)"

import os
from tarfile import *
from string import strip
from shutil import copyfile
from urllib2 import urlopen
from time import localtime
import ConfigParser

# Gets current date
today = '%d-%d-%d' % localtime()[:3]

# Configuration
config = os.path.expanduser('~/.config/notefinder/notefinder.ini')

def reloadDb():
    # Reading configuration
    global path
    if os.path.exists(config) and os.path.isfile(config):
        try:
            configuration = ConfigParser.ConfigParser()
            configuration.read(config)
            profile = configuration.get("general", "profile")
            path = unicode(configuration.get("profiles", profile), 'utf')
        except:
            path = os.path.expanduser('~/.notes')
    else:
        path = os.path.expanduser('~/.notes')


    # Creating directories
    if not os.path.exists(path):
        os.mkdir(path)

    if not os.path.exists(os.path.join(path, 'date')):
        os.makedirs(os.path.join(path, 'date'))
    
    if not os.path.exists(os.path.join(path, 'tag')):
        os.makedirs(os.path.join(path, 'tag'))

    if not os.path.exists(os.path.join(path, 'bookmarks')):
        os.makedirs(os.path.join(path, 'bookmarks'))

    if not os.path.exists(os.path.join(path, 'todo')):
        os.makedirs(os.path.join(path, 'todo'))

    if not os.path.exists(os.path.join(path, 'images')):
        os.makedirs(os.path.join(path, 'images'))

    # Changing working directory
    os.chdir(path)

reloadDb()

class Storage(object):
    def __init__(self):
        self.path = 'date'
        self.tagPath = 'tag'
        self.bookmarkPath = 'bookmarks'
        self.todoPath = 'todo'
        self.imagesPath = 'images'

    def getDates(self):
        ''' Returns list of all dates '''
        return os.listdir(self.path)
    
    def getDatesNumber(self):
        ''' Returns total dates number '''
        return len(self.getDates())

    def getTags(self):
        ''' Returns list of all tags '''
        return os.listdir(self.tagPath)

    def getTagsNumber(self):
        ''' Returns total tags number '''
        return len(self.getTags())

    def getNotes(self):
        ''' Returns list of all notes '''
        notes = []
        for date in self.getDates():
            notes += Date(date).getNotes()
        return notes
    
    def getNotesNumber(self):
        ''' Returns total notes number '''
        return len(self.getNotes())
    
    def getBookmarks(self):
        ''' Returns bookmarked notes '''
        return os.listdir(self.bookmarkPath)
    
    def getBookmarksNumber(self):
        ''' Returns number of bookmarked notes '''
        return len(self.getBookmarks())

    def export(self, filePath, bookmarks=True, tags = True, searches = True,
            images = True, history = True, TODOs=True):
        ''' Exports whole db to one tar-ed file '''
        tar = TarFile(name=filePath, mode="w")
        tar.add(self.path)
        if tags:
            tar.add(self.tagPath)
        if bookmarks:
            tar.add(self.bookmarkPath)
        if TODOs:
            tar.add(self.todoPath)
        if images:
            tar.add(self.imagesPath)
        if searches:
            try:
                tar.add("searches.dat")
            except:
                pass
        if history:
            try:
                tar.add("history.dat")
            except:
                pass
        tar.close()
    
    def importDb(self, filePath):
        ''' Imports tar-ed file to db '''
        tar = TarFile(name=filePath, mode="r")
        tar.extractall()
    
    def addImage(self, filePath, name):
        ''' Import image to gallery '''
        copyfile(filePath, os.path.join(self.imagesPath, name))
    
    def deleteImage(self, name):
        ''' Delete image '''
        os.remove(os.path.join(self.imagesPath, name))
    
    def getImages(self):
        return os.listdir(self.imagesPath)

class TODO(object):
    def __init__(self):
        self.path = Storage().todoPath
    
    def getNotes(self):
        ''' Return all TODO-list entries '''
        return os.listdir(self.path)
    
    def addEntry(self, note):
        ''' Add/remove entry to/from TODO list '''
        if not (note in self.getNotes()):
            try:
                os.symlink(os.path.join("..", Note(note).getPath()), os.path.join(self.path, note))
            except:
                # Hack for filesystems without symbolic links support >_<
                fakelink = open(os.path.join(self.path, note), 'w')
                fakelink.close()
        else:
            try:
                os.unlink(os.path.join(self.path, note))
            except:
                os.remove(os.path.join(self.path, note))
            

class Note(object):
    def __init__(self, name):
        self.name = name
    
    def exists(self):
        ''' Returns True if note exists and False if not'''
        if self.getPath() is None:
            return False
        else:
            return True

    def getDate(self):
        ''' Returns creation date of selected note '''
        date = None
        for date in Storage().getDates():
            if Date(date).hasNote(self.name):
                return date
    
    def getPath(self):
        ''' Returns file path for selected note '''
        path = None
        if self.getDate() != None:
            path = os.path.join(Storage().path, self.getDate(), self.name)
        return path
    
    def getSize(self):
        ''' Returns file size '''
        return os.path.getsize(self.getPath())/1024.0

    def getText(self):
        ''' Returns full text for selected note '''
        text = None
        if self.exists():
            file = open(self.getPath(), 'r')
            text = ''.join(file.readlines())
            file.close()
        return text
    
    def getLength(self):
        ''' Returns length of entry text '''
        return len(self.getText())

    def getTags(self):
        ''' Returns tags applied to selected note '''
        tags = [tag for tag in Storage().getTags() if Tag(tag).hasNote(self.name)]
        return tags
    
    def findRelated(self):
        ''' Finds related notes '''
        notes = [note for note in Search([self.name]).getNotes()]
        for tag in self.getTags():
            for note in Search([tag]).getNotes():
                if note not in notes:
                    notes.append(note)
        return notes

    def remove(self):
        ''' Removes note and date/tag dirs if empty '''
        if self.exists():
            date = Date(self.getDate())
            tags = self.getTags()
            os.remove(self.getPath())
            date.remove()
            for tag in tags:
                try:
                    os.unlink(os.path.join(Storage().tagPath, tag, self.name))
                except:
                    # Hack for filesystems without symbolic links support >_<
                    os.remove(os.path.join(Storage().tagPath, tag, self.name))
                if Tag(tag).getNumber() == 0:
                    # Removing tag if it has no any note
                    Tag(tag).remove()
            if self.name in Storage().getBookmarks():
                try:
                    os.unlink(os.path.join(Storage().bookmarkPath, self.name))
                except:
                    # Hack for filesystems without symbolic links support >_<
                    os.remove(os.path.join(Storage().bookmarkPath, self.name))
            if self.name in TODO().getNotes():
                try:
                    os.unlink(os.path.join(TODO().path, self.name))
                except:
                    # Hack for filesystems without symbolic links support >_<
                    os.remove(os.path.join(TODO().path, self.name))
    
    def rename(self, name):
        new = Note(name)
        new.write(self.getText())
        new.tag(self.getTags())
        if self.name in Storage().getBookmarks():
            new.bookmark()
        if self.name in TODO().getNotes():
            TODO().addEntry(name)
        self.remove()

    def write(self, text):
        ''' Writes some text to note '''
        if not self.exists():
            Date(today).create()
            file = open(os.path.join(Storage().path, today, self.name), 'w')
            file.write(text)
            file.close()
        else:
            file = open(self.getPath(), 'w')
            file.write(text)
            file.close()

    def tag(self, tags):
        ''' Applies user given tags to note '''
        if self.exists():
            src = self.getTags()
            for tag in tags:
                if tag not in src:
                    Tag(tag).create()
                    try:
                        os.symlink(os.path.join("../..", self.getPath()), os.path.join(Storage().tagPath, tag, self.name))
                    except:
                        # Hack for filesystems without symbolic links support >_<
                        fakelink = open(os.path.join(Storage().tagPath, tag, self.name), 'w')
                        fakelink.close()
            for tag in src:
                if tag not in tags:
                    try:
                        os.unlink(os.path.join(Storage().tagPath, tag, self.name))
                    except:
                        # Hack for filesystems without symbolic links support >_<
                        os.remove(os.path.join(Storage().tagPath, tag, self.name))
                    Tag(tag).remove()
    
    def bookmark(self):
        ''' Bookmarks/unbookmarks note '''
        if not (self.name in Storage().getBookmarks()):
            try:
                os.symlink(os.path.join("..", self.getPath()), os.path.join(Storage().bookmarkPath, self.name))
            except:
                # Hack for filesystems without symbolic links support >_<
                fakelink = open(os.path.join(Storage().bookmarkPath, self.name), 'w')
                fakelink.close()
        else:
            try:
                os.unlink(os.path.join(Storage().bookmarkPath, self.name))
            except:
                # Hack for filesystems without symbolic links support >_<
                os.remove(os.path.join(Storage().bookmarkPath, self.name))

    def import_(self, Path, link=False):
        ''' Imports specified local or remote file as new note '''
        if not self.exists():
            if Path.startswith("http://"):
                    page = urlopen(Path)
                    text = ''.join(page.readlines())
                    self.write(text)
            else:
                if os.path.exists(Path) and os.path.isfile(Path):
                    Date(today).create()
                    if link:
                        os.symlink(Path, os.path.join(Storage().path, today, self.name))
                    else:
                        copyfile(Path, os.path.join(Storage().path, today, self.name))
   
    def export(self, filePath):
        ''' Exports note to specfied location '''
        if self.exists():
            if not os.path.exists(filePath):
                copyfile(self.getPath(), filePath)

class Date(object):
    def __init__(self, date):
        self.value = date

    def year(self):
        ''' Return date's year '''
        return self.value.split('-')[0]

    def month(self):
        ''' Returns date's month '''
        return self.value.split('-')[1]
    
    def day(self):
        ''' Returns date's day '''
        return self.value.split('-')[2]

    def exists(self):
        ''' Returns True if date exists and False if not '''
        if self.value in Storage().getDates():
            return True
        else:
            return False

    def getPath(self):
        ''' Returns path for selected date '''
        path = None
        if self.exists():
            path = os.path.join(Storage().path, self.value)
        return path

    def getNotes(self):
        ''' Returns list that contains notes created at selected date '''
        notes = []
        if self.exists():
            notes = os.listdir(self.getPath())
        return notes
    
    def getNumber(self):
        ''' Returns number of notes created at selected date '''
        return len(self.getNotes())
    
    def remove(self):
        ''' Removes selected date if there are no notes created at it '''
        if self.exists() and self.getNumber() == 0:
            os.rmdir(self.getPath())
    
    def create(self):
        ''' Creates specified date '''
        dir = os.path.join(Storage().path, self.value)
        if not self.exists():
            os.mkdir(dir)
    
    def hasNote(self, name):
        ''' Returns True if specified note dated by selected date '''
        if name in self.getNotes():
            return True
        else:
            return False

class Tag(object):
    def __init__(self, tag):
        self.value = tag
    
    def exists(self):
        ''' Returns True if tag exists and False if not '''
        if self.value in Storage().getTags():
            return True
        else:
            return False
    
    def getPath(self):
        ''' Returns path for selected tag '''
        path = None
        if self.exists():
            path = os.path.join(Storage().tagPath, self.value)
        return path

    def getNotes(self):
        ''' Returns list that contains notes tagged with selected tag '''
        notes = []
        if self.exists():
            notes = os.listdir(self.getPath())
            if '.directory' in notes:
                notes.remove(".directory")
        return notes
    
    def getNumber(self):
        ''' Returns number of notes tagged with selected tag '''
        return len(self.getNotes())

    def remove(self):
        ''' Removes selected tag '''
        if self.exists():
            for i in self.getNotes():
                os.remove(os.path.join(self.getPath(), i))
            os.rmdir(self.getPath())
    
    def create(self):
        ''' Creates specified tag '''
        dir = os.path.join(Storage().tagPath, self.value)
        if not self.exists():
            os.mkdir(dir)

    def hasNote(self, name):
        ''' Returns True if specified note is tagged with tag '''
        if name in self.getNotes():
            return True
        else:
            return False

class Text(object):
    def __init__(self, text):
        self.value = text

    def getNotes(self):
        ''' Returns notes that contains specified text fragment '''
        notes = []
        if self.value != None:
            for note in Storage().getNotes():
                if self.value in Note(note).getText():
                    notes.append(note)
        return notes
    
    def getNumber(self):
        ''' Returns number of notes that contain specified text fragment '''
        return len(self.getNotes())

    def getKeywords(self):
        ''' Returns list containing 5 most commonly used words in text '''
        keywords = []
        words = {}
        text = self.value
        for word in text.split(" "):
            if len(word) >= 5 and not (word.startswith('<') or word.endswith('>')):
                try:
                    words[word] += 1
                except KeyError:
                    words[word] = 1
        pairs = words.items()
        pairs.sort(lambda a, b: b[1]-a[1]) 
        for p in pairs[:5]:
            keywords.append(p[0])
        return keywords

   
class Search(object):
    def __init__(self, items):
        self.items = items
    
    def getNotes(self):
        ''' Returns list that contains notes matching all search items '''
        full_list = []
        for item in self.items:
            if item.startswith("date:"):
                notes = Date(item[5:]).getNotes()
            elif item.startswith("year:"):
                notes = []
                for date in Storage().getDates():
                    if Date(date).year() == item[5:]:
                        notes += Date(date).getNotes()
            elif item.startswith("month:"):
                notes = []
                for date in Storage().getDates():
                    if Date(date).month() == item[6:]:
                        notes += Date(date).getNotes()
            elif item.startswith("day:"):
                notes = []
                for date in Storage().getDates():
                    if Date(date).day() == item[4:]:
                        notes += Date(date).getNotes()
            elif item.startswith("tag:"):
                notes = Tag(item[4:]).getNotes()
            elif item.startswith("text:"):
                notes = Text(item[5:]).getNotes()
            elif item.startswith("title:"):
                notes = []
                name = item[6:]
                if Note(name).exists():
                    notes.append(name)
            else:
                notes = Date(item).getNotes() + Tag(item).getNotes() + Text(item).getNotes()
                if Note(item).exists():
                    if item not in notes:
                        notes.append(item)
            full_list.append(notes)
        list_intersection = lambda list: set(list[0]) if len(list) == 1 else set(list[0]).intersection(list_intersection(list[1::]))
        return list(list_intersection(full_list))
    
    def getNumber(self):
        ''' Returns number of notes matching all search items '''
        return len(self.getNotes())
