#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import sys
import tarfile

from PyQt4 import QtGui, QtCore

from libnofi import Storage
from db import Ui_DatabaseDialog
from message_impl import Message

class DB(QtGui.QDialog):
    def __init__(self, parent = None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_DatabaseDialog()
        self.ui.setupUi(self)
        
        # Default choice
        self.ui.importRadioButton.setChecked(True)
        
        # Dialogs
        self.message = Message()
        
        # Connections
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.do)
    
    def import_(self):
        ''' Importing existing database '''
        fileDialog = QtGui.QFileDialog()
        fileDialog.setWindowTitle(self.trUtf8("Import database"))
        fileDialog.exec_()
        file = unicode(str(fileDialog.selectedFiles()[0].toUtf8()), 'utf').encode(sys.getfilesystemencoding())
        if os.path.exists(file) and os.path.isfile(file):
            try:
                Storage().importDb(file)
                self.accept()
            except tarfile.ReadError:
                self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to import specified files.</b></p>\
                    <p>This error may occur if:<ul><li>Specified file isn't NoteFinder database</li><li>It's corrupted</li></ul></p></body></html>"))
                self.message.show()
                self.message.raise_()
                self.message.activateWindow()
            except OSError:
                self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to import specified file.<b></p>\
                    <p>This error may occur if:<ul><li>Disk space is not available</li><li>You have no write access to NoteFinder database</li></ul></p>\
                    <p>You can solve this by:<ul><li>Freeing some disk space</li><li>Setting correct file permissions for database<br>\
                    using your file manager or <b>chmod</b> UNIX utility</li></ul></p></body></html>"))
                self.message.show()
                self.message.raise_()
                self.message.activateWindow()

    def export(self):
        ''' Exporting current database '''
        fileDialog = QtGui.QFileDialog()
        fileDialog.setWindowTitle(self.trUtf8("Export database"))
        fileDialog.setAcceptMode(fileDialog.AcceptSave)
        fileDialog.exec_()
        file = unicode(str(fileDialog.selectedFiles()[0].toUtf8()), 'utf').encode(sys.getfilesystemencoding())
        try:
            Storage().export(
                file, bookmarks=self.ui.bookmarkCheckBox.isChecked(),
                tags = self.ui.tagsCheckBox.isChecked(),
                searches = self.ui.searchesCheckBox.isChecked(),
                images = self.ui.imagesCheckBox.isChecked(),
                history = self.ui.historyCheckBox.isChecked(),
                TODOs=self.ui.todoCheckBox.isChecked()
                )
        except:
            self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to export database.<b></p>\
                <p>This error may occur if:<ul><li>Disk space is not available</li><li>You have no write access to path you specified</li></ul></p>\
                <p>You can solve this by:<ul><li>Freeing some disk space</li><li>Setting correct file permissions for specified<br>\
                using your file manager or <b>chmod</b> UNIX utility</li><li>Selecting other path</li></ul></p></body></html>"))
            self.message.show()
            self.message.raise_()
            self.message.activateWindow()

    def do(self):
        ''' Selecting action to do '''
        if self.ui.importRadioButton.isChecked():
            self.import_()
        elif self.ui.exportRadioButton.isChecked():
            self.export()
        self.accept()