#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys
from datetime import datetime

from PyQt4 import QtGui, QtCore

from libnofi import Note, today
import notefinder_rc
from editor import Ui_EditorDialog
from editor_widget_impl import Widget
from message_impl import Message

class Editor(QtGui.QDialog):
    def __init__(self, parent = None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_EditorDialog()
        self.ui.setupUi(self)
        self.ui.tabWidget.clear()
        
        # Opened notes list
        self.openedNotes = []
        
        # Adding tab corner widgets
        self.ui.newButton = QtGui.QToolButton(self)
        self.ui.newButton.setIcon(QtGui.QIcon(":/tab_new.png"))
        self.ui.newButton.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.ui.newButton.setShortcut(self.trUtf8("Ctrl+N"))
        self.ui.closeButton = QtGui.QToolButton(self)
        self.ui.closeButton.setIcon(QtGui.QIcon(":/tab_remove.png"))
        self.ui.closeButton.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.ui.closeButton.setShortcut(self.trUtf8("Ctrl+W"))
        box = QtGui.QDialogButtonBox()
        box.addButton(self.ui.closeButton, QtGui.QDialogButtonBox.ActionRole)
        box.addButton(self.ui.newButton, QtGui.QDialogButtonBox.ActionRole)
        self.ui.tabWidget.setCornerWidget(box)
        
        # Switching tabs
        self.nextTabAction = QtGui.QAction(self)
        self.connect(self.nextTabAction, QtCore.SIGNAL("triggered()"), self.nextTab)
        self.nextTabAction.setShortcut(self.trUtf8("Alt+Right"))

        self.prevTabAction = QtGui.QAction(self)
        self.connect(self.prevTabAction, QtCore.SIGNAL("triggered()"), self.prevTab)
        self.prevTabAction.setShortcut(self.trUtf8("Alt+Left"))

        # Dialogs
        self.message = Message()
        
        # Connections
        self.connect(self.ui.newButton, QtCore.SIGNAL("clicked()"), self.new)
        self.connect(self.ui.saveButton, QtCore.SIGNAL("clicked()"), self.saveNote)
        self.connect(self.ui.quitButton, QtCore.SIGNAL("clicked()"), self.closeAll)
        self.connect(self.ui.closeButton, QtCore.SIGNAL("clicked()"), self.closeTab)
    
    def closeEvent(self, event):
        self.hide()
        event.ignore()

    def nextTab(self):
        ''' Switch to next tab '''
        self.ui.tabWidget.setCurrentIndex(self.ui.tabWidget.currentIndex() + 1)
    
    def prevTab(self):
        ''' Switch to previous tab '''
        self.ui.tabWidget.setCurrentIndex(self.ui.tabWidget.currentIndex() - 1)
    
    def new(self):
        ''' Creating new tab '''
        tab = Widget()
        tab.ui.nameEdit.setEnabled(True)
        name = 'New note %s' % (datetime.strftime(datetime.today(),
            '%Y-%m-%d %H-%M-%S'))
        tab.ui.nameEdit.setText(self.trUtf8(name))
        tab.ui.nameEdit.setFocus()
        tab.ui.date.setText(self.trUtf8(today))
        self.ui.tabWidget.addTab(tab, name)
        self.ui.tabWidget.setCurrentIndex(self.ui.tabWidget.indexOf(tab))
        self.ui.tabWidget.setTabIcon(self.ui.tabWidget.indexOf(tab),
            QtGui.QIcon(":/note.png"))
        if self.isHidden():
            self.show()


    def saveNote(self):
        ''' Saving currently selected tab's note '''
        currentWidget = self.ui.tabWidget.currentWidget()
        name = str(currentWidget.ui.nameEdit.text().toUtf8())
        text = currentWidget.ui.textEdit.toPlainText().toUtf8()
        tags = currentWidget.getTags()
        if not ("/" in name) and not ("\\" in name) and not ("?" in name) and not ('"' in name)\
            and not (":" in name) and not ("<" in name) and not (">" in name) and not ("|" in name):
            try:
                Note(name).write(text)
                Note(name).tag(tags)
                self.ui.tabWidget.setTabText(self.ui.tabWidget.indexOf(currentWidget),
                    self.trUtf8(name))
                currentWidget.ui.nameEdit.setEnabled(False)
                currentWidget.refresh()
                self.history.add(name)
                self.emit(QtCore.SIGNAL("entrySaved()"))
            except:
                self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to save entry.<b></p>\
                    <p>This error may occur if:<ul><li>Disk space is not available</li><li>You have no write access to NoteFinder database</li></ul></p>\
                    <p>You can solve this by:<ul><li>Freeing some disk space</li><li>Setting correct file permissions for database<br>\
                    using your file manager or <b>chmod</b> UNIX utility</li></ul></p></body></html>"))
                self.message.show()
                self.message.raise_()
                self.message.activateWindow()
        else:
            self.message.ui.message.setText(self.trUtf8('<html><body><p><b>Failed to save entry.<b></p>\
                <p>Entry name cannot contain following characters: / | \ < > " ? :</p></body></html>'))
            self.message.show()
            self.message.raise_()
            self.message.activateWindow()
    
    def closeAll(self):
        ''' Closing all tabs '''
        self.ui.tabWidget.clear()
        self.close()
        self.openedNotes = []
    
    def closeTab(self):
        ''' Closing current tab '''
        note = str(self.ui.tabWidget.currentWidget().ui.nameEdit.text().toUtf8())
        if note in self.openedNotes:
            self.openedNotes.remove(note)
        self.ui.tabWidget.removeTab(self.ui.tabWidget.currentIndex())
