#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys
from datetime import datetime

from PyQt4 import QtGui, QtCore

from libnofi import Note, Tag, Text, Storage
from pycreole import wiki2html

import notefinder_rc
from editor_widget import Ui_Form
from message_impl import Message
from images_impl import Images

class Widget(QtGui.QWidget):
    def __init__(self, parent = None):
        QtGui.QWidget.__init__(self, parent)
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        self.refresh()

        # Dialogs
        self.message = Message()
        self.images = Images()

        # Connections
        self.connect(self.ui.addTagButton, QtCore.SIGNAL("clicked()"), self.addTag)
        self.connect(self.ui.delTagButton, QtCore.SIGNAL("clicked()"), self.delTag)
        self.connect(self.ui.autoTagButton, QtCore.SIGNAL("clicked()"), self.autoTag)
        self.connect(self.ui.textEdit, QtCore.SIGNAL("textChanged()"), self.generateHtml)
        self.connect(self.ui.boldButton, QtCore.SIGNAL("toggled(bool)"), self.bold)
        self.connect(self.ui.italicButton, QtCore.SIGNAL("toggled(bool)"), self.italic)
        self.connect(self.ui.underlineButton, QtCore.SIGNAL("toggled(bool)"), self.underline)
        self.connect(self.ui.highlightButton, QtCore.SIGNAL("toggled(bool)"), self.highlight)
        self.connect(self.ui.imageButton, QtCore.SIGNAL("clicked()"), self.images.show)
        self.connect(self.ui.timeButton, QtCore.SIGNAL("clicked()"), self.insertTime)
        self.connect(self.ui.tagEdit.lineEdit(), QtCore.SIGNAL("returnPressed()"), self.addTag)
        self.connect(self.ui.tagsList, QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem *)"), self.editTag)
        self.connect(self.images.ui.insertButton, QtCore.SIGNAL("clicked()"), self.insertImage)         
        self.connect(self.ui.textBrowser, QtCore.SIGNAL("anchorClicked(const QUrl)"), self.openUrl)

    def openUrl(self, url):
        QtGui.QDesktopServices().openUrl(url)
        
    def bold(self):
        ''' Makes font bold '''
        self.ui.textEdit.insertPlainText("**")

    def italic(self):
        ''' Makes font italic '''
        self.ui.textEdit.insertPlainText("//")
        
    def underline(self):
        ''' Makes font underlined '''
        self.ui.textEdit.insertPlainText("__")
    
    def highlight(self):
        ''' Makes font underlined '''
        self.ui.textEdit.insertPlainText("%%")
    
    def insertImage(self):
        ''' Inserts image '''
        image = self.images.currentImage()
        if not image is None:
            self.ui.textEdit.insertPlainText(self.trUtf8("{{images/%s}}" % (image)))
    
    def insertTime(self):
        ''' Inserts timestamp '''
        self.ui.textEdit.insertPlainText(datetime.strftime(datetime.today(),'%Y-%m-%d, %H:%M:%S'))
        
    def addTag(self):
        '''' Adding entered tag to list '''
        line = str(self.ui.tagEdit.currentText().toUtf8())
        if line == '':
            self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to assign tag.<b></p>\
                <p>Entered tag is empty</p></body></html>"))
            self.message.show()
        elif line in self.getTags():
            self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to assign tag.<b></p>\
                <p>Tag is already in list</p></body></html>"))
            self.message.show()
        elif '/' in line  or '\\' in line or '"' in line or '?' in line or ':' in line or "<" in line or ">" in line or "|" in line:
            self.message.ui.message.setText(self.trUtf8('<html><body><p><b>Failed to assign tag.<b></p>\
                <p>Tag may not contain following characters: / | \ < > " ? :</p></body></html>'))
            self.message.show()
        else:
            tag = Tag(line)
            item = QtGui.QListWidgetItem(self.ui.tagsList)
            item.setText(self.trUtf8(tag.value))
            item.setIcon(QtGui.QIcon(":/tag.png"))
            item.setToolTip(self.trUtf8("<html><body>Tag: %s<br>Notes (%i): %s</body></html>"
                % (tag.value, tag.getNumber(), ', '.join(tag.getNotes()))))
            self.ui.tagEdit.lineEdit().clear()

    def delTag(self):
        ''' Removing current item from list '''
        index = self.ui.tagsList.currentRow()
        self.ui.tagsList.takeItem(index)
    
    def getTags(self):
        ''' Gets currently added tags list '''
        tagNumber = self.ui.tagsList.count()
        taglist = []
        for i in xrange(0, tagNumber):
            taglist.append(str(self.ui.tagsList.item(i).text().toUtf8()))
        return taglist
    
    def autoTag(self):
        ''' Autotagging '''
        text = str(self.ui.textBrowser.toPlainText().toUtf8())
        tags = Text(text).getKeywords()
        for i in tags:
            if (i not in self.getTags()) and ('/' not in i) and ('\\' not in i)\
            and ('"' not in i) and ('?' not in i) and (':' not in i) and ("<" not in i) and (">" not in i)\
                and ("|" not in i):
                tag = Tag(i)
                item = QtGui.QListWidgetItem(self.ui.tagsList)
                item.setText(self.trUtf8(tag.value))
                item.setIcon(QtGui.QIcon(":/tag.png"))
                item.setToolTip(self.trUtf8("<html><body>Tag: %s<br>Notes (%i): %s</body></html>"
                    % (tag.value, tag.getNumber(), ', '.join(tag.getNotes()))))
        
    def editTag(self):
        index = self.ui.tagsList.currentRow()
        item = self.ui.tagsList.item(index)
        self.ui.tagsList.openPersistentEditor(item)

    def generateHtml(self):
        ''' Generating HTML from Wiki on the fly
        PyCreole library is used.
        Author - Peter Hoffmann <tosh54@gmail.com>
        License - GNU GPL v2
        Big thanks! ;)
        '''
        text = str(self.ui.textEdit.toPlainText().toUtf8())
        if text.startswith("<html>"):
            self.ui.textBrowser.setHtml(self.trUtf8(text))
        else:
            html = wiki2html(unicode(text, 'utf'))
            self.ui.textBrowser.setHtml(self.trUtf8(html))
    
    def refresh(self):
        ''' Cleaning interface up '''
        self.ui.tagEdit.clear()
        for tag in Storage().getTags():
            self.ui.tagEdit.addItem(self.trUtf8(tag))
            self.ui.tagEdit.setEditText(self.trUtf8(""))
