#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import pickle

from libnofi import Note

class History(object):
    ''' History class '''
    def __init__(self):
        ''' Initializing '''
        self.list = []
        self.historyFile = "history.dat"
        
        # Loading history from file
        if os.path.exists(self.historyFile):
            fd = open(self.historyFile, 'r')
            try:
                self.list = pickle.load(fd)
                fd.close()
            except:
                pass
    
    def add(self, note):
        ''' Add entry to history '''
        if note in self.list:
            self.list.remove(note)
        self.list.append(note)
        self.syncToFile()

    def syncToFile(self):
        fd = open(self.historyFile, 'w')
        pickle.dump(self.list, fd)
        fd.close()
    
    def getNotes(self):
        history = [i for i in self.list]
        history.reverse()
        return history
    
    def delete(self, note):
        ''' Delete entry from list '''
        if note in self.list:
            self.list.remove(note)
        self.syncToFile()
    
    def refresh(self):
        self.list = [i for i in self.getNotes() if Note(i).exists()]
    