#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import sys

from PyQt4 import QtCore, QtGui

from libnofi import Storage
import notefinder_rc
from images import Ui_ImagesDialog
from add_img_impl import AddImage
from message_impl import Message

class Images(QtGui.QDialog):
    def __init__(self, parent = None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_ImagesDialog()
        self.ui.setupUi(self)
        
        # Dialogs
        self.add_img = AddImage()
    
        # Connections
        self.connect(self.ui.addButton, QtCore.SIGNAL("clicked()"), self.add_img, QtCore.SLOT("show()"))
        self.connect(self.ui.deleteButton, QtCore.SIGNAL("clicked()"), self.delete)
        self.connect(self.add_img.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.refresh)
        
        # Refreshing
        self.refresh()
    
    def refresh(self):
        self.ui.imagesList.clear()
        for i in Storage().getImages():
            item = QtGui.QListWidgetItem(self.ui.imagesList)
            item.setIcon(QtGui.QIcon(self.trUtf8(os.path.join(Storage().imagesPath, i))))
            item.setText(self.trUtf8(i))
            item.setToolTip("<html><body><img src='%s'></body></html>" % self.trUtf8((os.path.join(Storage().imagesPath, i))))

    
    def currentImage(self):
        image = None
        index = self.ui.imagesList.currentRow()
        item = self.ui.imagesList.item(index)
        if not item is None:
            image = str(item.text().toUtf8())
        return image

    def delete(self):
        image = self.currentImage()
        print image
        if not image is None:
            Storage().deleteImage(image)
        self.refresh()