#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys
from datetime import datetime
from inspect import isclass
import pickle
import ConfigParser

from PyQt4 import QtCore, QtGui

try:
    from libnofi import *
except:
    sys.exit("Something went wrong. Terminating.")
from pycreole import wiki2html
from history import History
import notefinder_rc
from main import Ui_MainWindow
from plugin import Plugin
from editor_impl import Editor
from add_tag_impl import AddTag
from delete_tag_impl import DeleteTag
from editor_widget_impl import Widget
from delete_impl import Delete
from rename_impl import Rename
from importDlg_impl import Import
from db_impl import DB
from notebooks_impl import Notebooks
from about_impl import About
from message_impl import Message

class NoteAction(QtGui.QAction):
    def __init__(self, note, parent = None):
        QtGui.QAction.__init__(self, parent)
        self.setIcon(QtGui.QIcon(":/history.png"))
        self.setText(self.trUtf8(note))
        self.connect(self, QtCore.SIGNAL("triggered()"), self.open)
        self.main = parent

    def open(self):
        self.main.open(str(self.text().toUtf8()))
        

class MainWindow(QtGui.QMainWindow):
    def __init__(self, parent = None):
        QtGui.QMainWindow.__init__(self, parent)

        # Dialogues
        self.ui = Ui_MainWindow()
        self.notebooks = Notebooks()
        self.about = About()
        self.delete = Delete()
        self.deleteTag = DeleteTag()
        self.addTag = AddTag()
        self.rename = Rename()
        self.editor = Editor()
        self.Import = Import()
        self.message = Message()
        self.DB = DB()
        self.ui.setupUi(self)
        self.ui.splitter.setSizes([250, 1030])
        
        # Search widget
        self.spacerWidget = QtGui.QWidget()
        self.horizontalLayout = QtGui.QHBoxLayout(self.spacerWidget)
        spacerItem = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.ui.toolBar.addWidget(self.spacerWidget)
        self.searchLabel = QtGui.QLabel()
        self.searchLabel.setPixmap(QtGui.QPixmap(":/search.png"))
        self.ui.toolBar.addWidget(self.searchLabel)
        self.searchEdit = QtGui.QLineEdit()
        self.ui.toolBar.addWidget(self.searchEdit)
        self.saveSearchAction = QtGui.QAction(self)
        self.saveSearchAction.setText("Save search")
        self.ui.toolBar.addAction(self.saveSearchAction)

        # History object
        self.history = History()
        self.editor.history = self.history
        
        # Searches
        self.searchesFile = "searches.dat"
        self.loadSearches()

        # Tray icon
        self.tray = QtGui.QSystemTrayIcon(self)
        self.headerAction = QtGui.QWidgetAction(self)
        self.headerFrame = QtGui.QFrame()
        self.headerFrame.setFrameShape(QtGui.QFrame.StyledPanel)
        self.headerAction.setDefaultWidget(self.headerFrame)
        self.headerLayout = QtGui.QHBoxLayout()
        self.headerLayout.setMargin(3)
        self.headerLayout.setSpacing(5)
        self.headerFrame.setLayout(self.headerLayout)
        self.headerLabel = QtGui.QLabel()
        self.headerLabel.setPixmap(QtGui.QPixmap(":/note.png"))
        self.headerLayout.insertWidget(-1, self.headerLabel, 0)
        self.headerText = QtGui.QLabel("<b>NoteFinder</b>")
        self.headerLayout.insertWidget(-1, self.headerText, 20)
        self.menu = QtGui.QMenu()
        self.submenu = QtGui.QMenu()
        self.submenu.setTitle("Recent entries")
        self.submenu.setIcon(QtGui.QIcon(":/history.png"))
        self.menu.addAction(self.headerAction)
        self.menu.addMenu(self.submenu)
        self.menu.addSeparator()
        self.menu.addAction(self.ui.actionNew)
        self.menu.addAction(self.ui.actionNotebooks)
        self.menu.addSeparator()
        self.pluginsMenu = QtGui.QMenu()
        self.pluginsMenu.setTitle("Plugins")
        self.menu.addMenu(self.pluginsMenu)
        self.menu.addSeparator()
        self.menu.addAction(self.ui.actionExit)
        self.tray.setContextMenu(self.menu)
        self.tray.setIcon(QtGui.QIcon(":/note.png"))
        self.tray.show()
        self.connect(self.tray, QtCore.SIGNAL("activated(QSystemTrayIcon::ActivationReason)"), self.showHide)
        
        self.recreateTrayMenu()
        
        # Connections
        self.connect(self.ui.actionNew, QtCore.SIGNAL("triggered()"), self.editor.new)
        self.connect(self.ui.actionDelete, QtCore.SIGNAL("triggered()"), self.removeNote)
        self.connect(self.ui.actionRename, QtCore.SIGNAL("triggered()"), self.renameNote)
        self.connect(self.ui.actionAddTag, QtCore.SIGNAL("triggered()"), self.addTag, QtCore.SLOT("show()"))
        self.connect(self.ui.actionDeleteTag, QtCore.SIGNAL("triggered()"), self.delTag)
        self.connect(self.ui.actionCreate_desktop_shortcut, QtCore.SIGNAL("triggered()"), self.createShortcut)
        self.connect(self.ui.actionChange_Icon, QtCore.SIGNAL("triggered()"), self.changeIcon)
        self.connect(self.ui.actionDelete_Icon, QtCore.SIGNAL("triggered()"), self.deleteIcon)
        self.connect(self.ui.actionImport, QtCore.SIGNAL("triggered()"), self.Import, QtCore.SLOT("show()"))
        self.connect(self.ui.actionExport, QtCore.SIGNAL("triggered()"), self.export)
        self.connect(self.ui.actionExport_to_PDF, QtCore.SIGNAL("triggered()"), self.pdf)
        self.connect(self.ui.actionE_Mail, QtCore.SIGNAL("triggered()"), self.email)
        self.connect(self.ui.actionBookmark, QtCore.SIGNAL("triggered()"), self.bookmark)
        self.connect(self.ui.actionTODO, QtCore.SIGNAL("triggered()"), self.todo)
        self.connect(self.ui.actionBookmarks, QtCore.SIGNAL("triggered()"), self.showBookmarks)
        self.connect(self.ui.actionTODOS, QtCore.SIGNAL("triggered()"), self.showTODOS)
        self.connect(self.ui.actionHistory, QtCore.SIGNAL("triggered()"), self.showHistory)
        self.connect(self.ui.actionNotebooks, QtCore.SIGNAL("triggered()"), self.notebooks, QtCore.SLOT("show()"))
        self.connect(self.ui.actionDB, QtCore.SIGNAL("triggered()"), self.DB, QtCore.SLOT("show()"))
        self.connect(self.ui.actionFindRelated, QtCore.SIGNAL("triggered()"), self.findRelated)
        self.connect(self.ui.actionAll, QtCore.SIGNAL("triggered()"), self.showAll)
        self.connect(self.ui.actionToday, QtCore.SIGNAL("triggered()"), self.showToday)
        self.connect(self.ui.actionDeleteSelectedSearch, QtCore.SIGNAL("triggered()"), self.deleteSearch)
        self.connect(self.ui.actionAbout, QtCore.SIGNAL("triggered()"), self.about.show)
        self.connect(self.ui.notesList, QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem *)"), self.selectNote)
        self.connect(self.ui.calendarWidget, QtCore.SIGNAL("selectionChanged()"), self.openDate)
        self.connect(self.ui.tagsList, QtCore.SIGNAL("itemClicked(QListWidgetItem *)"), self.openTag)
        self.connect(self.ui.searchesList, QtCore.SIGNAL("itemClicked(QListWidgetItem *)"), self.openSearch)
        self.connect(self.searchEdit, QtCore.SIGNAL("textChanged(const QString &)"), self.search)
        self.connect(self.editor, QtCore.SIGNAL("entrySaved()"),  self.refresh)
        self.connect(self.Import.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.refresh)
        self.connect(self.delete.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.refresh)
        self.connect(self.rename.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.refresh)
        self.connect(self.deleteTag.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.refresh)
        self.connect(self.DB, QtCore.SIGNAL("accepted()"), self.refresh)
        self.connect(self.addTag.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.refresh)
        self.connect(self, QtCore.SIGNAL("historyChanged()"), self.recreateTrayMenu)
        self.connect(self.ui.actionDeleteFromBatch, QtCore.SIGNAL("triggered()"), self.deleteFromBatch)
        self.connect(self.notebooks, QtCore.SIGNAL("profileChanged()"), self.reload)
        self.connect(self.saveSearchAction, QtCore.SIGNAL("triggered()"), self.saveSearch)
        self.connect(self.ui.actionExit, QtCore.SIGNAL("triggered()"), QtGui.qApp.quit)

        # Creating note's context menu
        self.ui.notesList.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self.ui.notesList.addAction(self.ui.actionBookmark)
        self.ui.notesList.addAction(self.ui.actionTODO)
        self.ui.notesList.addAction(self.ui.actionExport)
        self.ui.notesList.addAction(self.ui.actionE_Mail)
        self.ui.notesList.addAction(self.ui.actionFindRelated)
        self.ui.notesList.addAction(self.ui.actionRename)
        self.ui.notesList.addAction(self.ui.actionDelete)
        
        # Creating tag's context menu
        self.ui.tagsList.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self.ui.tagsList.addAction(self.ui.actionAddTag)
        self.ui.tagsList.addAction(self.ui.actionDeleteTag)
        self.ui.tagsList.addAction(self.ui.actionChange_Icon)
        self.ui.tagsList.addAction(self.ui.actionDelete_Icon)
        self.ui.tagsList.addAction(self.ui.actionCreate_desktop_shortcut)

        # Creating batch's contet menu
        self.ui.batchList.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self.ui.batchList.addAction(self.ui.actionDeleteFromBatch)
        
        # Saved searches' context menu
        self.ui.searchesList.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
        self.ui.searchesList.addAction(self.ui.actionDeleteSelectedSearch)

        # Bold font
        self.boldFont = QtGui.QFont()
        self.boldFont.setBold(True)

        # Startup
        self.parameter = TODO()
        self.refresh()
        self.refreshSearches()
        
        # Drag&Drop
        self.ui.notesList.setDragEnabled(True)
        self.ui.batchList.setAcceptDrops(True)
        
        # Win32-specific
        if sys.platform == 'win32':
            self.ui.actionCreate_desktop_shortcut.setEnabled(False)
        
        
  
    def recreateTrayMenu(self):
        # Recreating tray menu
        self.submenu.clear()
        for note in self.history.getNotes():
            self.submenu.addAction(NoteAction(note, self))

    def closeEvent(self, event):
        self.hide()
        event.ignore()

    def showHide(self, reason):
        if reason == QtGui.QSystemTrayIcon.Trigger:
            if self.isHidden():
                self.show()
                self.activateWindow()
            else:
                self.hide()

    def refreshBatch(self):
        self.ui.batchList.clear()
    
    def getBatch(self):
        number = self.ui.batchList.count()
        batchlist = []
        for i in xrange(0, number):
            batchlist.append(str(self.ui.batchList.item(i).text().toUtf8()))
        return batchlist

   
    def deleteFromBatch(self):
        index = self.ui.batchList.currentRow()
        self.ui.batchList.takeItem(index)
    
    def reload(self):
        reloadDb()
        self.history = History()
        self.loadSearches()
        self.refreshSearches()
        self.refresh()

    def refresh(self):
        ''' Refreshing UI at startup and after changes '''
        # Refreshing tags list
        self.ui.tagsList.clear()
        for i in Storage().getTags():
            tag = Tag(i)
            item = QtGui.QListWidgetItem(self.ui.tagsList)
            item.setText(self.trUtf8(tag.value))
            dir_desc = os.path.join(tag.getPath(), ".directory")
            if os.path.exists(dir_desc):
                config = ConfigParser.ConfigParser()
                config.read(dir_desc)
                if config.has_option("Desktop Entry", "Icon"):
                    "Yes, exists"
                    icon = unicode(config.get("Desktop Entry", "Icon"), 'utf')
                else:
                    icon = ":/tag.png"
            else:
                icon = ":/tag.png"
            item.setIcon(QtGui.QIcon(icon))
            item.setToolTip(self.trUtf8
                ("<html><body>Tag: %s<br>Notes (%i): %s</p></body></html>" %
                (tag.value, tag.getNumber(), ', '.join(tag.getNotes()))))
        
        # Refreshing history
        self.history.refresh()
        
        # Refreshing batch
        self.refreshBatch()
        
        # Displaying entries
        if self.parameter != None:
            try:
                list = self.parameter.getNotes()
            except:
                list = self.parameter()
            self.display(list)
            self.emit(QtCore.SIGNAL("historyChanged()"))

        # Adding search completer items
        self.completer = QtGui.QCompleter([self.trUtf8(i) for i in Storage().getDates()] + [self.trUtf8(i) for i in Storage().getTags()])
        self.searchEdit.setCompleter(self.completer)

        # Refreshing summary
        self.ui.statusbar.showMessage((self.trUtf8
            ("Today is %s    Tags: %i    Notes: %i" %
            (today,  Storage().getTagsNumber(), Storage().getNotesNumber()))))

    def display(self, list):
        ''' Displaying created by some searches notes list '''
        self.ui.notesList.clear()
        number = len(list)
        if number == 1:
            self.ui.numberLabel.setText(self.trUtf8("%i result" % (number)))
        else:
            self.ui.numberLabel.setText(self.trUtf8("%i results" % (number)))
        for i in list:
            try:
                note = Note(i)
                item = QtGui.QListWidgetItem(self.ui.notesList)
                item.setText(self.trUtf8(note.name))
                item.setFont(self.boldFont)
                if note.name in Storage().getBookmarks():
                    if note.name in TODO().getNotes():
                        item.setIcon(QtGui.QIcon(":/bookmark-todo.png"))
                    else:
                        item.setIcon(QtGui.QIcon(":/bookmark.png"))
                elif note.name in TODO().getNotes():
                    item.setIcon(QtGui.QIcon(":/todo.png"))
                else:
                    item.setIcon(QtGui.QIcon(":/note.png"))
                text = unicode(note.getText(), 'utf')[:100]
                if text.startswith("<html>"):
                    html = text
                else:
                    html = wiki2html(text)
                item.setToolTip(self.trUtf8(
                    "<html><body>Title: %s<br>Date: %s<br>Tags: <b>%s</b><br>Text: %s...<br>Length: %i characters\
                    <br>Size: %f KB</body></html>"
                    % (note.name, note.getDate(), ', '.join(note.getTags()), html, note.getLength(), note.getSize())))
            except:
                self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to open entry %s<b></p>\
                    <p>This error may occur if:<ul><li>Something is wrong with file's encoding.</li><li>You have no read access to NoteFinder database</li></ul></p>\
                    <p>You can solve this by:<ul><li>Fixing file's encoding with any text editor</li><li>Setting correct file permissions for database<br>\
                    using your file manager or <b>chmod</b> UNIX utility</li></ul></p>\
                    <p>Entry path: %s</p></body></html>" % (i, note.getPath())))
                self.message.show()
    
    def displayBatch(self):
        self.ui.batchList.clear()
        for note in self.getBatch():
            item = QtGui.QListWidgetItem(self.ui.batchList)
            item.setText(self.trUtf8(note))
            item.setIcon(QtGui.QIcon(":/note.png"))

    def openDate(self):
        ''' Displaying notes created at selected date '''
        selectedDate = self.ui.calendarWidget.selectedDate()
        year = str(selectedDate.year())
        month = str(selectedDate.month())
        day = str(selectedDate.day())
        self.parameter = Date(year+'-'+month+'-'+day)
        self.setWindowTitle(self.trUtf8("NoteFinder - %s" % (self.parameter.value)))
        self.display(self.parameter.getNotes())

    def openTag(self):
        ''' Displaying notes tagged with selected tag '''
        index = self.ui.tagsList.currentRow()
        item = self.ui.tagsList.item(index)
        self.parameter = Tag(str(item.text().toUtf8()))
        self.setWindowTitle(self.trUtf8("NoteFinder - %s" % (self.parameter.value)))
        self.display(self.parameter.getNotes())
    
    def openSearch(self):
        ''' Displaying selected saved search '''
        index = self.ui.searchesList.currentRow()
        item = self.ui.searchesList.item(index)
        text = str(item.text().toUtf8())
        if ' ' in text:
            self.parameter = Search(text.split[' '])
        else:
            self.parameter = Search([str(item.text().toUtf8())])
        self.setWindowTitle(self.trUtf8("NoteFinder - %s" % (" ".join(self.parameter.items))))
        self.display(self.parameter.getNotes())    
    
    def showAll(self):
        ''' Displaying all notes '''
        self.parameter = Storage()
        self.setWindowTitle("NoteFinder - All entries")
        self.display(self.parameter.getNotes())
    
    def showBookmarks(self):
        ''' Show bookmarks '''
        self.parameter = Storage().getBookmarks
        self.setWindowTitle(self.trUtf8("NoteFinder - Bookmarks"))
        self.display(self.parameter())
    
    def showTODOS(self):
        ''' Show TODO entries '''
        self.parameter = TODO()
        self.setWindowTitle(self.trUtf8("NoteFinder - TODO"))
        self.display(self.parameter.getNotes())
    
    def showHistory(self):
        ''' Show history '''
        self.parameter = self.history
        self.setWindowTitle(self.trUtf8("NoteFinder - History"))
        self.display(self.parameter.getNotes())

    def showToday(self):
        ''' Displaying notes selected this day '''
        self.parameter = Date(today)
        self.setWindowTitle(self.trUtf8("NoteFinder - Today"))
        self.display(self.parameter.getNotes())

    def search(self):
        ''' Displaying search results '''
        items = [item for item in str(self.searchEdit.text().toUtf8()).split(' ')]
        self.parameter = Search(items)
        self.setWindowTitle(self.trUtf8("NoteFinder - %s" % (" ".join(self.parameter.items))))
        self.display(self.parameter.getNotes())
    
    def currentNote(self):
        note = None
        index = self.ui.notesList.currentRow()
        item = self.ui.notesList.item(index)
        if not item is None:
            note = str(item.text().toUtf8())
        return note

    def findRelated(self):
        ''' Displaying notes related to selected note '''
        name = self.currentNote()
        if not name is None:
            note = Note(name)
            self.parameter = note.findRelated
            self.setWindowTitle(self.trUtf8("NoteFinder - Entries related to %s" % (note.name)))
            self.display(self.parameter())
        
    def open(self, name):
        ''' Opening selected note '''       
        # Checking if not already opened
        if name not in self.editor.openedNotes:
            tab = Widget()
            tab.ui.tabWidget.setCurrentIndex(0)
        
            # Adding to history
            self.history.add(name)
            self.emit(QtCore.SIGNAL("historyChanged()"))

            # Disabling namebar
            tab.ui.nameEdit.setEnabled(False)
    
            # Setting name
            tab.ui.nameEdit.setText(self.trUtf8(name))
    
            # Creating Note instance
            note = Note(name)
            tab.ui.date.setText(self.trUtf8(note.getDate()))
            text = note.getText()
            tab.ui.textEdit.setPlainText(self.trUtf8(note.getText()))
            
            # HTML or Wiki
            if text.startswith("<html>"):
                html = text
            else:
                html = wiki2html(unicode(text, "utf"))
            tab.ui.textBrowser.setHtml(self.trUtf8(html))
                
            # Displaying tags
            for tag in note.getTags():
                item = QtGui.QListWidgetItem(tab.ui.tagsList)
                item.setText(self.trUtf8(tag))
                item.setIcon(QtGui.QIcon(":/tag.png"))
                item.setToolTip(self.trUtf8("<html><body>Tag: %s<br>Notes (%i): %s</body></html>"
                    % (tag, Tag(tag).getNumber(), ', '.join(Tag(tag).getNotes()))))

            # Showing dialogue
            tab.ui.nameEdit.setEnabled(False)
            self.editor.ui.tabWidget.addTab(tab, "")
            self.editor.openedNotes.append(name)
            self.editor.ui.tabWidget.setCurrentIndex(self.editor.ui.tabWidget.indexOf(tab))
            self.editor.ui.tabWidget.setTabText(self.editor.ui.tabWidget.indexOf(tab),
                self.trUtf8(name))
            self.editor.ui.tabWidget.setTabIcon(self.editor.ui.tabWidget.indexOf(tab),
                QtGui.QIcon(":/note.png"))
            self.editor.show()
            self.editor.raise_()
            self.editor.activateWindow()
        
        else:
            if self.editor.isHidden():
                self.editor.show()
    
    def selectNote(self):
        self.open(self.currentNote())
    
    def clearSearch(self):
        ''' Clears searchbar and notes list '''
        self.ui.searchEdit.clear()
        self.ui.notesList.clear()

    def removeNote(self):
        ''' Removing note '''
        name = self.currentNote()
        if not name is None:
            self.delete.note = name
            self.delete.show()
    
    def renameNote(self):
        ''' Rename note '''
        name = self.currentNote()
        if not name is None:
            self.rename.note = name
            self.rename.show()
    
    def bookmark(self):
        ''' Add bookmark '''
        name = self.currentNote()
        if not name is None:
            note = Note(name)
            try:
                note.bookmark()
            except OSError:
                self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to bookmark entry.<b></p>\
                    <p>This error may occur if:<ul><li>Disk space is not available</li><li>You have no write access to NoteFinder database</li></ul></p>\
                    <p>You can solve this by:<ul><li>Freeing some disk space</li><li>Setting correct file permissions for database<br>\
                    using your file manager or <b>chmod</b> UNIX utility</li></ul></p></body></html>"))
                self.message.show()
                self.message.raise_()
                self.message.activateWindow()
            self.refresh()
    
    def todo(self):
        ''' Add/delete to/from TODO '''
        name = self.currentNote()
        if not name is None:
            try:
                TODO().addEntry(name)
            except:
                self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to add selected entry to TODO list.<b></p>\
                    <p>This error may occur if:<ul><li>Disk space is not available</li><li>You have no write access to NoteFinder database</li></ul></p>\
                    <p>You can solve this by:<ul><li>Freeing some disk space</li><li>Setting correct file permissions for database<br>\
                    using your file manager or <b>chmod</b> UNIX utility</li></ul></p></body></html>"))
                self.message.show()
        self.refresh()
   
    def export(self):
        ''' Exporting to file '''
        name = self.currentNote()
        if not name is None:
            note = Note(name)
            fileDialog = QtGui.QFileDialog()
            fileDialog.setWindowTitle(self.trUtf8("Export entry"))
            fileDialog.setAcceptMode(fileDialog.AcceptSave)
            fileDialog.exec_()
            file = unicode(str(fileDialog.selectedFiles()[0].toUtf8()), 'utf').encode(sys.getfilesystemencoding())
            try:
                note.export(file)
            except:
                self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to export selected entry.<b></p>\
                    <p>This error may occur if:<ul><li>Disk space is not available</li><li>You have no write access to specified path</li></ul></p>\
                    <p>You can solve this by:<ul><li>Freeing some disk space</li><li>Setting correct file permissions for specified path<br>\
                    using your file manager or <b>chmod</b> UNIX utility</li></ul></p></body></html>"))
                self.message.show()
                self.message.raise_()
                self.message.activateWindow()
    
    def email(self):
        name = self.currentNote()
        if not name is None:
            note = Note(name)
            url = QtCore.QUrl(self.trUtf8("mailto:?subject=%s&body=%s" % (note.name, note.getText())))
            QtGui.QDesktopServices.openUrl(url)
    
    def pdf(self):
        name = self.currentNote()
        if not name is None:
            note = Note(name)
            fileDialog = QtGui.QFileDialog()
            fileDialog.setWindowTitle(self.trUtf8("Export entry"))
            fileDialog.setAcceptMode(fileDialog.AcceptSave)
            fileDialog.exec_()
            file = unicode(str(fileDialog.selectedFiles()[0].toUtf8()), 'utf').encode(sys.getfilesystemencoding())
            try:
                document = QtGui.QTextDocument()
                document.setHtml(wiki2html(unicode(note.getText(), 'utf')))
                printer = QtGui.QPrinter()
                printer.setPageSize(QtGui.QPrinter.A4)
                printer.setOutputFormat(QtGui.QPrinter.PdfFormat)
                printer.setOutputFileName(file)
                document.print_(printer)

            except:
                self.message.ui.message.setText(self.trUtf8("<html><body><p><b>Failed to export selected entry.<b></p>\
                    <p>This error may occur if:<ul><li>Disk space is not available</li><li>You have no write access to specified path</li></ul></p>\
                    <p>You can solve this by:<ul><li>Freeing some disk space</li><li>Setting correct file permissions for specified path<br>\
                    using your file manager or <b>chmod</b> UNIX utility</li></ul></p></body></html>"))
                self.message.show()
                self.message.raise_()
                self.message.activateWindow()
    
    def delTag(self):
        ''' Deleting tag '''
        index = self.ui.tagsList.currentRow()
        item = self.ui.tagsList.item(index)
        if not item is None:
            tag = str(item.text().toUtf8())
            self.deleteTag.tag = tag
            self.deleteTag.show()
    
    def createShortcut(self):
        index = self.ui.tagsList.currentRow()
        item = self.ui.tagsList.item(index)
        if not item is None:
            tag = str(item.text().toUtf8())
            fd = open(os.path.join(os.path.expanduser("~/Desktop"), tag + ".desktop"), 'w')
            fd.write("[Desktop Entry]\n\
                Encoding=UTF-8\n\
                Icon=folder\n\
                Type=Link\n\
                URL=%s\n" % (os.path.join(os.path.abspath(os.curdir), Tag(tag).getPath())))
            fd.close()

    def changeIcon(self):
        index = self.ui.tagsList.currentRow()
        item = self.ui.tagsList.item(index)
        if not item is None:
            tag = str(item.text().toUtf8())
            tag_obj = Tag(tag)
            fd = open(os.path.join(tag_obj.getPath(), ".directory"), 'w')
            fileDialog = QtGui.QFileDialog()
            fileDialog.setWindowTitle(self.trUtf8("Import entry"))
            fileDialog.setFilter(self.trUtf8("Image Files (*.png *.jpg *.bmp *.ico)"))
            fileDialog.exec_()
            file = unicode(str(fileDialog.selectedFiles()[0].toUtf8()), 'utf')
            config = ConfigParser.ConfigParser()
            config.add_section("Desktop Entry")
            config.set("Desktop Entry", "Icon", file)
            config.write(fd)
            fd.close()
            self.refresh()
    
    def deleteIcon(self):
        index = self.ui.tagsList.currentRow()
        item = self.ui.tagsList.item(index)
        if not item is None:
            tag = str(item.text().toUtf8())
            tag_obj = Tag(tag)
            f = os.path.join(tag_obj.getPath(), ".directory")
            fileDialog = QtGui.QFileDialog()
            os.remove(f)
            self.refresh()
    
    def saveSearch(self):
        ''' Saves search '''
        searchText = str(self.searchEdit.text().toUtf8())
        if searchText != '':
            if searchText not in self.searches:
                self.searches.append(searchText)
                fd = open(self.searchesFile, 'w')
                pickle.dump(self.searches, fd)
                fd.close()
                self.refreshSearches()
            else:
                self.message.ui.message.setText(self.trUtf8("<p><b>Unable to save current search</b></p><p>\
                    It's already saved</p>"))
                self.message.show()
        else:
            self.message.ui.message.setText(self.trUtf8("<p><b>Unable to save current search</b></p><p>\
                    Empty search</p>"))
            self.message.show()
    
    def deleteSearch(self):
        index = self.ui.searchesList.currentRow()
        item = self.ui.searchesList.item(index)
        if item != None:
            text = str(item.text().toUtf8())
            self.searches.remove(text)
            fd = open(self.searchesFile, 'w')
            pickle.dump(self.searches, fd)
            fd.close()
            self.refreshSearches()

    def refreshSearches(self):
        self.ui.searchesList.clear()
        for i in self.searches:
            item = QtGui.QListWidgetItem(self.ui.searchesList)
            item.setText(self.trUtf8(i))
            item.setIcon(QtGui.QIcon(":/search.png"))
    
    def loadSearches(self):
        try:
            fd = open(self.searchesFile, 'r')
            self.searches = pickle.load(fd)
            fd.close()
        except:
            self.searches = []

app = QtGui.QApplication(sys.argv)
translator = QtCore.QTranslator()
locale = str(QtCore.QLocale.system().name())
translator.load("notefinder_" + locale + ".qm", ":/")
app.installTranslator(translator)
main = MainWindow()

# Load plugins
try:
    import pkg_resources
    for ep in pkg_resources.iter_entry_points('notefinder.plugins'):
        try:
            plug = ep.load()
            if isclass(plug):
                if issubclass(plug, Plugin):
                    plug(main).load(main)
        except:
                pass
except ImportError:
    main.ui.menuPlugins.setEnabled(False)
    
main.show()
main.tray.setVisible(True)
main.tray.show()
sys.exit(app.exec_())