#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import ConfigParser

from PyQt4 import QtGui, QtCore

from libnofi import config
from notebook import Ui_NotebookDialog
from message_impl import Message

class Notebook(QtGui.QDialog):
    def __init__(self, parent = None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_NotebookDialog()
        self.ui.setupUi(self)
        
        # Dialogs
        self.message = Message()
        
        # Connections
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.save)
        self.connect(self.ui.openButton, QtCore.SIGNAL("clicked()"), self.open)
    
    def refresh(self):
        ''' Clean up UI '''
        self.ui.pathEdit.clear()
        self.ui.nameEdit.clear()
    
    def save(self):
        ''' Save profile '''
        name = str(self.ui.nameEdit.text().toUtf8())
        path = str(self.ui.pathEdit.text().toUtf8())
        if not os.path.exists(config):
            os.makedirs(os.path.split(config)[0])
        configuration = ConfigParser.ConfigParser()
        configuration.read(config)
        if not configuration.has_section("profiles"):
            configuration.add_section("profiles")
        if name != '' and path != '':
            if os.path.isdir(path) or not os.path.exists(path):
                configuration.set("profiles", name, path)
                fd = open(config, 'w')
                configuration.write(fd)
                fd.close()
            else:
                self.message.ui.message.setText("Specified path exists and isn't directory")
                self.message.show()
                self.message.raise_()
                self.message.activateWindow()    
        else:
            self.message.ui.message.setText("Enter profile name and file path")
            self.message.show()
            self.message.raise_()
            self.message.activateWindow()  
        self.accept()

    def open(self):
        ''' Opens file chooser dialog '''
        fileDialog = QtGui.QFileDialog()
        fileDialog.setWindowTitle(self.trUtf8("Change location"))
        fileDialog.setFileMode(fileDialog.Directory)
        fileDialog.exec_()
        path = fileDialog.selectedFiles()[0]
        self.ui.pathEdit.setText(path)