#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import ConfigParser

from PyQt4 import QtGui, QtCore

from libnofi import config
import notefinder_rc
from notebooks import Ui_NotebooksDialog
from notebook_impl import Notebook
from message_impl import Message

class Notebooks(QtGui.QDialog):
    def __init__(self, parent = None):
        QtGui.QDialog.__init__(self, parent)
        self.ui = Ui_NotebooksDialog()
        self.ui.setupUi(self)
        self.refresh()
        
        # Dialogs
        self.notebook = Notebook()
        
        # Connections
        self.connect(self.ui.addButton, QtCore.SIGNAL("clicked()"), self.new)
        self.connect(self.ui.editButton, QtCore.SIGNAL("clicked()"), self.edit)
        self.connect(self.ui.deleteButton, QtCore.SIGNAL("clicked()"), self.delete)
        self.connect(self.notebook, QtCore.SIGNAL("accepted()"), self.refresh)
        self.connect(self.ui.buttonBox, QtCore.SIGNAL("accepted()"), self.save)

    def closeEvent(self, event):
        self.hide()
        event.ignore()
        
    def refresh(self):
        ''' Refresh profiles list '''
        self.ui.profilesList.clear()
        configuration = ConfigParser.ConfigParser()
        if os.path.exists(config):
            configuration.read(config)
        if configuration.has_section("profiles"):
            for i in configuration.items("profiles"):
                try:
                    item = QtGui.QListWidgetItem(self.ui.profilesList)
                    item.setText(self.trUtf8(i[0]))
                    item.setIcon(QtGui.QIcon(":/x-office-address-book.png"))
                    item.setToolTip(self.trUtf8(i[1]))
                except:
                    pass

    def new(self):
        ''' Create new profile '''
        self.notebook.refresh()
        self.notebook.show()
        self.notebook.raise_()
        self.notebook.activateWindow()

    def edit(self):
        ''' Edit profile '''
        index = self.ui.profilesList.currentRow()
        item = self.ui.profilesList.item(index)
        if not item is None:
            profile = str(item.text().toUtf8())
            configuration = ConfigParser.ConfigParser()
            configuration.read(config)
            self.notebook.refresh()
            self.notebook.ui.nameEdit.setText(self.trUtf8(profile))
            self.notebook.ui.pathEdit.setText(self.trUtf8(configuration.get("profiles", profile)))
            self.notebook.show()
            self.notebook.raise_()
            self.notebook.activateWindow()
    
    def delete(self):
        index = self.ui.profilesList.currentRow()
        item = self.ui.profilesList.item(index)
        print item
        if not item is None:
            profile = str(item.text().toUtf8())
            configuration = ConfigParser.ConfigParser()
            configuration.read(config)
            configuration.remove_option("profiles", profile)
            fp = open(config, 'w')
            configuration.write(fp)
            fp.close()
            del configuration
            self.refresh()

    def save(self):
        ''' Saves configuration '''
        index = self.ui.profilesList.currentRow()
        item = self.ui.profilesList.item(index)
        if item != None:
            profile = str(item.text().toUtf8())
            if not os.path.exists(config):
                os.makedirs(os.path.split(config)[0])
            configuration = ConfigParser.ConfigParser()
            configuration.read(config)
            if not configuration.has_section("general"):
                configuration.add_section("general")
            configuration.set("general", "profile", profile)
            fp = open(config, 'w')
            configuration.write(fp)
            fp.close()
            del configuration
        self.emit(QtCore.SIGNAL("profileChanged()"))
        self.close()