#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#       Copyright (C) 2007-2008 GFORGX, NoteFinder Team <gforgx@gmail.com>
#
#       Redistribution and use in source and binary forms, with or without
#       modification, are permitted provided that the following conditions are
#       met:
#
#       * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#       * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#       * Neither the name of the project NoteFinder and author's name nor the names of 
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
#       THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#      "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#       LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#       A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#       OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#       SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#       LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#       DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#       THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#       OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
import shutil

from notefinder import plugin, notefinder_rc
from PyQt4 import QtCore, QtGui
from libnofi import *
from pycreole import wiki2html

# CSS by DiOnX
# Begin ---
# Copyright (c) 2008 DiOnX <dr.onx@mail.ru>
css = "\
body{\n\
        padding: 0px;\n\
        font-family:	verdana;\n\
        BORDER:		0px solid;}\n\
a{\n\
        TEXT-DECORATION: underline;\n\
        font-family:	verdana;\n\
        font-size:	12px;\n\
        color:		'#000000';}\n\
A:hover{\n\
	TEXT-DECORATION:	none;\n\
	color:	A3CF62;}\n\
h1{\n\
	font-size:	24;}\n"
# --- End

class WebNotebook(plugin.Plugin):
    def __init__(self, parent = None):
        plugin.Plugin.__init__(self, parent)
        self.setText("Web Notebook")
        self.setIcon(QtGui.QIcon(":/import_web.png"))
        self.main = None
        
    def do(self):
        # Show filechooser
        fileDialog = QtGui.QFileDialog()
        fileDialog.setWindowTitle(self.trUtf8("Select path"))
        fileDialog.setFileMode(fileDialog.Directory)
        fileDialog.exec_()
        path = unicode(str(fileDialog.selectedFiles()[0].toUtf8()), 'utf').encode(sys.getfilesystemencoding())
        
        # Creating directories
        if not os.path.exists(os.path.join(path, "notes")):
            os.makedirs(os.path.join(path, "notes"))
        if not os.path.exists(os.path.join(path, "dates")):
            os.makedirs(os.path.join(path, "dates"))
        if not os.path.exists(os.path.join(path, "tags")):
            os.makedirs(os.path.join(path, "tags"))
        
        # Writing CSS
        cssFile = open(os.path.join(path, "style.css"), 'w')
        cssFile.write(css)
        cssFile.close()
        
        # Creating index
        index = "<html>\n<head>\n<title>Web Notebook</title>\n\
            <link rel='stylesheet' type='text/css' href='style.css'>\n\
            <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n\
            </head>\n\
            <body>\n<h1>NoteFinder Web Notebook</h1>\n\
            <ul>\n<li><a href='all.html'>All entries</a></li>\n\
            <li><a href='dates.html'>Dates</a></li>\n\
            <li><a href = 'tags.html'>Tags</a></li>\n\
            <li><a href='todo.html'>TODO list</a></li>\n</ul>\n</body></html>"
        indexFile = open(os.path.join(path, "index.html"), 'w')
        indexFile.write(index)
        indexFile.close()
        
        # Creating notes
        for note in Storage().getNotes():
            html = "<html>\n<head>\n<title>%s</title>\n\
                <link rel='stylesheet' type='text/css' href='../style.css'>\n\
                <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n\
                </head>\n\
                <body>\nDate: <a href = '../dates/%s.html'>%s</a>\n<br>\nTags: " % (note, Note(note).getDate(), Note(note).getDate())
            for tag in Note(note).getTags():
                html += "<a href = '../tags/%s.html'>%s</a> " % (tag, tag)
            html += "<br>\n %s</body>\n</html>" % (wiki2html(unicode(Note(note).getText(), "utf")))
            noteFile = open(os.path.join(path, "notes", note+".html"), 'w')
            noteFile.write(html)
            noteFile.close()
        
        # Copying images
        imagesPath = os.path.join(path, "notes/images")
        if os.path.exists(imagesPath):
            for i in os.listdir(imagesPath):
                os.remove(os.path.join(imagesPath, i))
            os.rmdir(imagesPath)
        shutil.copytree(Storage().imagesPath, os.path.join(path, "notes/images"))
        
        # Creating dates
        dateIndex = "<html>\n<head>\n<title>Dates</title>\n\
            <link rel='stylesheet' type='text/css' href='style.css'>\n\
            <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n\
            </head>\n\
            <body>\n<ul>\n"
        for date in Storage().getDates():
            dateIndex += "<li><a href = dates/%s.html>%s</a></li>\n" % (date, date)
            html = "<html>\n<head>\n<title>%s</title>\n\
                <link rel='stylesheet' type='text/css' href='../style.css'>\n\
                <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n\
                </head>\n\
                <body>\n<ul>\n" % (date)
            for note in Date(date).getNotes():
                html += "<li><a href = '../notes/%s.html'>%s</a></li>\n" % (note, note)
            html += "</ul>\n</body>\n</html>"
            dateFile = open(os.path.join(path, "dates", date+".html"), 'w')
            dateFile.write(html)
            dateFile.close()
        dateIndex += "</ul>\n</body>\n</html>\n"
        dateIndexFile = open(os.path.join(path, "dates.html"), 'w')
        dateIndexFile.write(dateIndex)
        dateIndexFile.close()
        
        # Creating tags
        tagIndex = "<html>\n<head>\n<title>Tags</title>\n\
            <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n\
            <link rel='stylesheet' type='text/css' href='style.css'>\n\
            </head>\n\
            <body>\n<ul>\n"
        for tag in Storage().getTags():
            tagIndex += "<li><a href = 'tags/%s.html'>%s</a></li>\n" % (tag, tag)
            html = "<html>\n<head>\n<title>%s</title>\n\
                <link rel='stylesheet' type='text/css' href='../style.css'>\n\
                <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n\
                </head>\n\
                <body>\n<ul>\n" % (tag)
            for note in Tag(tag).getNotes():
                html += "<li><a href = '../notes/%s.html'>%s</a></li>\n" % (note, note)
            html += "</ul></body></html>"
            tagFile = open(os.path.join(path, "tags", tag+".html"), 'w')
            tagFile.write(html)
            tagFile.close()
        tagIndex += "</ul>\n</body>\n</html>"
        tagIndexFile = open(os.path.join(path, "tags.html"), 'w')
        tagIndexFile.write(tagIndex)
        tagIndexFile.close()
        
        # Creating TODO
        todoIndex = "<html>\n<head>\n<title>TODO</title>\n\
            <link rel='stylesheet' type='text/css' href='style.css'>\n\
            <meta http-equiv='Content-Type' content='text/html; charset=utf-8' /></head>\n\
            <body>\n<ul>\n"
        for note in TODO().getNotes():
            todoIndex += "<li><a href = 'notes/%s.html'>%s</a></li>\n" % (note, note)
        todoIndex += "</ul>\n</body>\n</html>"
        todoIndexFile = open(os.path.join(path, "todo.html"), 'w')
        todoIndexFile.write(todoIndex)
        todoIndexFile.close()
        
        # Creating summary page
        all  = "<html>\n<head>\n<title>All entries</title>\n\
            <link rel='stylesheet' type='text/css' href='style.css'>\n\
            <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n\
            </head>\n\
            <body>\n<ul>\n"
        for note in Storage().getNotes():
            all += "<li><a href = 'notes/%s.html'>%s</a></li>\n" % (note, note)
        all += "</ul>\n</body>\n</html>"
        allFile = open(os.path.join(path, "all.html"), 'w')
        allFile.write(all)
        allFile.close()
        
        self.showMessage("Done!<br>You can access your Web notebook <a href=%s>here</a>" % (os.path.join(path, "index.html")))